﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CameraTarget : MonoBehaviour {

    [SerializeField] private Transform boatDrifter;
    [SerializeField] private float targetFocusRangeMin = 0;
    [SerializeField] private float targetFocusRangeMax = 0;
    [SerializeField] private float targetFocusFactor;
    [SerializeField] private Vector3 boatOffset;
    [SerializeField] private float maxWaveHeight;
    [SerializeField] private AnimationCurve followWaveCurve;

    private void Update() {
        List<Transform> targets = new List<Transform>();
        Wave.Instances.ForEach(x => targets.Add(x.transform));
        

        Dictionary<Transform, float> factors = new Dictionary<Transform, float>();
        foreach (Transform target in targets) {
            float distance = Vector3.Distance(boatDrifter.transform.position, target.transform.position);
            if (distance >= targetFocusRangeMax) { continue; }
            float factor = 1.0f - Mathf.Clamp01((distance - targetFocusRangeMin) / (targetFocusRangeMax - targetFocusRangeMin));

            Wave wave = target.GetComponent<Wave>();
            if (wave != null) {
                factor *= wave.Height / maxWaveHeight;
                factor *= followWaveCurve.Evaluate(wave.LifeProgression);
            }

            factor = EasingHelper.EaseInSine(factor);
            factors.Add(target, factor);
        }

        if (factors.Count == 0) {
            transform.position = boatDrifter.transform.position + GetBoatOffset();
            return;
        }

        Vector3 totalPosition = Vector3.zero;
        float totalFactor = 0.0f;
        foreach (KeyValuePair<Transform, float> pair in factors) {
            totalPosition += (pair.Key.transform.position) * pair.Value;
            totalFactor += pair.Value;
        }

        Vector3 weightPosition = (totalPosition + (boatDrifter.transform.position + GetBoatOffset())) / (totalFactor + 1.0f);

        transform.position = weightPosition;
    }

    private IEnumerator Start() {
        const float offsetMax = 15.0f;

        boatOffset.z = offsetMax;

        while (!Game.HasStarted) { yield return new WaitForEndOfFrame(); }

        //yield return new WaitForSeconds(0.0f);

        float time = 0.0f;
        float duration = 5.0f;

        while (time < duration && !Input.GetKey(KeyCode.Return)) {
            time += Time.deltaTime;
            float progress = time / duration;
            float eased = EasingHelper.EaseInOutSine(progress);
            boatOffset.z = (1.0f - eased) * offsetMax;

            yield return new WaitForEndOfFrame();
        }

        boatOffset.z = 0.0f;
    }

    private Vector3 GetBoatOffset() {
        return boatOffset;
    }

}
