﻿using Spine;
using Spine.Unity;
using System.Collections.Generic;
using UnityEngine;
using System;

public class Sailor : MonoBehaviour {

    [HideInInspector] public Workstation CurrentWorkstation;

    public bool HasDrowned { get; private set; }

    [SerializeField] private SkeletonAnimation skeletonAnimation;
    [SerializeField] private Drifter drifter;
    [SerializeField] private AudioSource drownAudio;
    [SerializeField] private List<string> skinNames;

    private Camera mainCamera;

    private float timeInWorkstation;

    public void Drown() {
        CurrentWorkstation.SetSailor(null);
        transform.SetParent(null);
        drifter.enabled = true;
        HasDrowned = true;
        Animate("drowning");
        GetComponent<SoundEmitter>().PlayOnce(drownAudio);
    }

    public void Animate(string animationName) {
        if (skeletonAnimation.AnimationState.GetCurrent(0).Animation.Name == animationName) { return; }
        skeletonAnimation.AnimationState.SetAnimation(0, animationName, true);
        skeletonAnimation.AnimationState.Tracks.ToArray()[0].TrackTime = UnityEngine.Random.value;
    }

    public void SetIsFlipped(bool flipX) {
        skeletonAnimation.Skeleton.FlipX = flipX;
    }

    public void Repriorities() {
        foreach (Workstation workstation in Workstation.Instances) {
            if (workstation.IsTaken) { continue; }
            if (CurrentWorkstation != null && workstation.Priority <= CurrentWorkstation.Priority) { continue; }
            workstation.SetSailor(this);
            timeInWorkstation = 0.0f;
            break;
        }
    }

    private void Awake() {
        mainCamera = FindObjectOfType<Camera>();
        drifter.enabled = false;
        
        skeletonAnimation.Skeleton.SetSkin(skinNames.GetRandom());
        skeletonAnimation.Skeleton.SetSlotsToSetupPose();
    }

    private void Start() {
        Repriorities();
    }

	private void Update () {
        transform.LookAt(mainCamera.transform.position, Vector3.up);
        Vector3 euler = transform.localRotation.eulerAngles;
        euler.x = 0.0f;
        euler.z = 0.0f;
        transform.localRotation = Quaternion.Euler(euler);


        timeInWorkstation += Time.deltaTime;
        if (CurrentWorkstation != null && timeInWorkstation > .5f) {
            Vector3 targetPosition = transform.parent.InverseTransformPoint(CurrentWorkstation.transform.position);
            if (transform.localPosition != targetPosition) {
                //Animate("l-walk");
                transform.localPosition = targetPosition;
            } else if (CurrentWorkstation != null) {
                //CurrentWorkstation.InitiateDefaultAnimation();
            } else { 
                //Animate("f-idle");
            }
        }
    }
}
