﻿using DG.Tweening;
using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Audio;

public class Storm : MonoBehaviour {

    public Action OnDone;

    public static Color BreakerColor;

    public IEnumerable<Wave> Waves { get { return waves; } }

    protected Color BackgroundColor { get { return backgroundColor; } }
    protected float TransitionDuration { get { return transitionDuration; } }

    private readonly List<Wave> waves = new List<Wave>();

    [Header("Transition")]
    [SerializeField] private float duration;

    [Header("Difficulty")]
    [SerializeField] private float intervalMin;
    [SerializeField] private float intervalMax;

    [Header("Waves")]
    [SerializeField] private List<Wave> wavePrefabs;

    [Header("Tone")]
    [SerializeField] private float transitionDuration;
    [SerializeField] private Color seaColorDark;
    [SerializeField] private Color seaColorBright;
    [SerializeField] private Color breakerColor;
    [SerializeField] private Color seaGradientColor;
    [SerializeField] private float seaGradientLength;
    [SerializeField] private Color backgroundColor;
    [SerializeField] private Color lighthouseColor;

    [Header("Music")]
    [SerializeField] private AudioMixerSnapshot audioSnapshot;

    private float age;

    public virtual void StartStorm(bool instant) {
        gameObject.SetActive(true);
        age = 0.0f;
        StartCoroutine("SpawnWaves");

        if (instant) {
            transitionDuration = 0.0f;
        }

        Material seaMaterial = FindObjectOfType<Sea>().GetComponent<Renderer>().material;
        seaMaterial.DOKill();
        seaMaterial.DOVector(seaColorBright, "_BrightColor", transitionDuration);
        seaMaterial.DOVector(seaColorDark, "_DarkColor", transitionDuration);
        seaMaterial.DOVector(seaGradientColor, "_GradientMultiply", transitionDuration);
        seaMaterial.DOFloat(seaGradientLength, "_GradientDistance", transitionDuration);
        seaMaterial.DOVector(backgroundColor, "_FogColor", transitionDuration);

        Camera mainCamera = Camera.main;
        mainCamera.DOKill();
        mainCamera.DOColor(backgroundColor, transitionDuration);

        Lighthouse lighthouse = FindObjectOfType<Lighthouse>();
        Renderer lighthouseRenderer = lighthouse.GetComponentInChildren<Renderer>();
        lighthouseRenderer.material.DOKill();
        lighthouseRenderer.material.DOColor(backgroundColor, "_FogColor", transitionDuration);
        lighthouseRenderer.material.DOColor(lighthouseColor, "_SolidColor", transitionDuration);
        
        EndingEnvironment environment = FindObjectOfType<EndingEnvironment>();
        Renderer[] envRenderers = environment.GetComponentsInChildren<Renderer>();
        foreach (Renderer r in envRenderers) {
            foreach (Material m in r.materials) {
                m.DOKill();
                m.DOColor(backgroundColor, "_FadeColor", transitionDuration);
                m.DOFloat(1.0f, "_FadeAmount", transitionDuration);
            }
        }

        CoroutineHelper.Delay(() => {
            if (audioSnapshot != null) {
                audioSnapshot.TransitionTo(transitionDuration);
            }
        }, .1f);

        BreakerColor = breakerColor;
    }

    public virtual void StopStorm() {
        StopAllCoroutines();
        gameObject.SetActive(false);
    }

    private void Update() {
        age += Time.deltaTime;
        if (age > duration) {
            if (OnDone != null) {
                OnDone();
            }
        }
    }

    private IEnumerator SpawnWaves() {
        yield return new WaitForSeconds(1.0f);

        while (true) {
            Wave prefab = wavePrefabs.GetRandom();
            if (prefab == null) { yield break; }

            Wave wave = Instantiate(prefab);
            wave.name = wave.name + " from " + gameObject.name;

            wave.OnDestroyCallback += OnWaveDestroy;
            waves.Add(wave);

            float interval = UnityEngine.Random.Range(intervalMin, intervalMax);
            yield return new WaitForSeconds(interval);
        }
    }

    private void OnWaveDestroy(Wave wave) {
        waves.RemoveAll(x => x == wave);
    }

}
