﻿using DG.Tweening;
using System.Collections.Generic;
using UnityEngine;

public class Workstation : MonoBehaviour {

    public readonly static List<Workstation> Instances = new List<Workstation>();

    public bool IsTaken { get { return targetSailor != null; } }
    public int Priority { get { return priority; } }

    [SerializeField] private int priority;
    [SerializeField] private string defaultAnimation;
    [SerializeField] private bool flipX;

    private Sailor targetSailor;

    public void SetSailor(Sailor sailor) {
        if (targetSailor != null) {
            targetSailor.CurrentWorkstation = null;
        }
        targetSailor = sailor;
        if (sailor != null) {
            if (sailor.CurrentWorkstation != null) {
                sailor.CurrentWorkstation.SetSailor(null);
            }
            sailor.CurrentWorkstation = this;
            sailor.Animate("l-walk");
            sailor.transform.DOKill();
            sailor.transform.DOLocalMove(sailor.transform.parent.InverseTransformPoint(transform.position), .5f)
                            .OnComplete(() => {
                                if (!string.IsNullOrEmpty(defaultAnimation)) {
                                    sailor.Animate(defaultAnimation);
                                }
                                sailor.SetIsFlipped(flipX);
                            });
        }
    }

    public void InitiateDefaultAnimation() {
        if (targetSailor == null) { return; }
        if (!string.IsNullOrEmpty(defaultAnimation)) {
            targetSailor.Animate(defaultAnimation);
        }
    }

    private void Awake() {
        Instances.Add(this);
        Instances.Sort(delegate (Workstation a, Workstation b) { return b.priority.CompareTo(a.priority); });
    }

    private void OnDestroy() {
        Instances.Remove(this);
    }

}
